@@ -1,10 +1,10 @@
 #! /bin/sh
  
-ln -s debian-rvm debian
-cp debian-rvm/changelog-orig debian-rvm/changelog
+ln -s debian-orig debian
+cp debian-orig/changelog-orig debian-orig/changelog
 
 ./get_svn_revision.sh
-#dch -v 17.4.0-`cat svn_revision` "New version"
+#dch -v 19.1.0-`cat svn_revision` "New version"
 
 #dpkg-buildpackage -rfakeroot
 
@@ -15,5 +15,5 @@ fakeroot debian/rules build
 fakeroot debian/rules binary
 
 dh_clean
-rm debian-rvm/changelog
+rm debian-orig/changelog
 rm debian
