@@ -1,5 +1,5 @@
 /*  smplayer, GUI front-end for mplayer.
-    Copyright (C) 2006-2017 Ricardo Villalba <rvm@users.sourceforge.net>
+    Copyright (C) 2006-2019 Ricardo Villalba <rvm@users.sourceforge.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -327,7 +327,7 @@ protected slots:
 	virtual void changeStyleSheet(QString style);
 #endif
 
-	void applyStyles();
+	virtual void applyStyles();
 
 	virtual void setTabletMode(bool);
 
@@ -492,6 +492,8 @@ protected:
 	MyAction * videoEqualizerAct;
 	MyAction * screenshotAct;
 	MyAction * screenshotsAct;
+	MyAction * screenshotWithSubsAct;
+	MyAction * screenshotWithNoSubsAct;
 #ifdef CAPTURE_STREAM
 	MyAction * capturingAct;
 #endif
@@ -519,9 +521,7 @@ protected:
 	MyAction * incAudioDelayAct;
 	MyAction * audioDelayAct; // Ask for delay
 	MyAction * extrastereoAct;
-#ifdef MPLAYER_SUPPORT
 	MyAction * karaokeAct;
-#endif
 	MyAction * volnormAct;
 #ifdef MPV_SUPPORT
 	MyAction * earwaxAct;
@@ -629,6 +629,7 @@ protected:
 	MyAction * nextWheelFunctionAct;
 
 	MyAction * showFilenameAct;
+	MyAction * showMediaInfoAct;
 	MyAction * showTimeAct;
 	MyAction * toggleDeinterlaceAct;
 
