@@ -1,5 +1,5 @@
 /*  smplayer, GUI front-end for mplayer.
-    Copyright (C) 2006-2014 Ricardo Villalba <rvm@users.sourceforge.net>
+    Copyright (C) 2006-2019 Ricardo Villalba <rvm@users.sourceforge.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 */
 
 #include "extensions.h"
+#include <QDebug>
 
 ExtensionList::ExtensionList() : QStringList()
 {
@@ -62,14 +63,16 @@ Extensions::Extensions()
            << "rm" << "swf"
            << "ts" << "rmvb" << "dvr-ms" << "m2t" << "m2ts" << "mts" << "rec" << "wtv"
            << "f4v" << "hdmov" << "webm" << "vp8"
-           << "bik" << "smk" << "m4b";
+           << "bik" << "smk" << "m4b" << "wtv"
+           << "part";
 
-	_audio << "mp3" << "ogg" << "oga" << "wav" << "wma" <<  "aac" << "ac3" << "dts" << "ra" << "ape" << "flac" << "thd" << "mka" << "m4a";
+	_audio << "mp3" << "ogg" << "oga" << "wav" << "wma" <<  "aac" << "ac3"
+           << "dts" << "ra" << "ape" << "flac" << "thd" << "mka" << "m4a" << "opus";
 
 	_subtitles << "srt" << "sub" << "ssa" << "ass" << "idx" << "txt" << "smi"
-               << "rt" << "utf" << "aqt";
+               << "rt" << "utf" << "aqt" << "vtt";
 
-	_playlist << "m3u" << "m3u8" << "pls";
+	_playlist << "m3u" << "m3u8" << "pls" << "xspf";
 
 	_multimedia = _video;
 	for (int n = 0; n < _audio.count(); n++) {
@@ -82,3 +85,21 @@ Extensions::Extensions()
 Extensions::~Extensions() {
 }
 
+QString Extensions::extensionFromUrl(const QString & url) {
+	//qDebug() << "Extensions::extensionFromUrl:" << url;
+
+	QString extension;
+	int pos = url.lastIndexOf(".");
+	if (pos != -1) {
+		extension = url.mid(pos+1).toLower();
+		// Check if extension contains a '?' and remove everything after it
+		pos = extension.lastIndexOf("?");
+		if (pos != -1) {
+			extension = extension.left(pos);
+		}
+	}
+
+	//qDebug() << "Extensions::extensionFromUrl: extension:" << extension;
+	return extension;
+}
+
